"use strict";

/**
 * @class {util4js.Object}
 */
var Object = $class({

  __static__: {
 
    /**
     * @memberOf {util4js.Object}
     * @static
     * @private
     * @field {Integer} __counter
     */
    __counter: 0,

    /**
     * @memberOf {util4js.Object}
     * @static
     * @public
     * @method equals
     * @param {Object} a
     * @param {Object} b
     */
    equals: function (a, b) {
      if (a === b) {
        return true;
      }
      if (!a || !b) {
        return false;
      }
      return a.equals(b);
    }
    
  },

  /**
   * @memberOf {util4js.Object}
   * @public
   * @constructor {util4js.Object}
   */
  __construct__: function (props) {
    this.__hashCode = ++Object.__counter;
  },

  /**
   * @memberOf {util4js.Object}
   * @public
   * @method equals
   * @param {Object} obj
   */
  equals: function (obj) {
    if (!obj) {
      return false;
    } 
    return this === obj; 
  },

  /**
   * @memberOf {util4js.Object}
   * @public
   * @method getHashCode
   */
  getHashCode: function () {
    return this.__hashCode;
  },

  /**
   * @memberOf {util4js.Object}
   * @public
   * @method toString
   * @returns {String}
   */
  toString: function () {
    return "util4js.Object";
  }

});

exports.Object = Object;

/**
 * @class {util4js.Exception}
 * @extends {util4js.Object}
 */
var Exception = $class({

  /**
   * @memberOf {util4js.Exception}
   * @public
   * @constructor {util4js.Exception}
   */
  __construct__: function () {
    this.__name = "Exception";
    this.__message = "An exception has occured";
    this.__innerException = null;
  },

  /**
   * @memberOf {util4js.Exception}
   * @public
   * @property {String} name
   */
  name: {
    get: function () {
      return this.__name;
    },
    set: function (value) {
      this.__name = value;
    }
  },

  /**
   * @memberOf {util4js.Exception}
   * @public
   * @property {String} message
   */
  message: {
    get: function () {
      return this.__message;
    },
    set: function (value) {
      this.__message = value;
    } 
  },

  /**
   * @memberOf {util4js.Exception}
   * @public
   * @property {Object} innerException
   */
  innerException: {
    get: function () {
      return this.__innerException;
    },
    set: function (value) {
      this.__innerException = value;
    }
  }, 

  /**
   * @memberOf {util4js.Exception}
   * @public
   * @overridden
   * @method toString
   * @retuns {String}
   */
  toString: function () {
    return this.name + ": " + this.message;
  }

}, Object);

exports.Exception = Exception;

/**
 * @class {util4js.ArgumentException}
 * @extends {util4js.Exception}
 */
var ArgumentException = $class({

  /**
   * @memberOf {util4js.ArgumentException}
   * @public
   * @construct {util4js.ArgumentException}
   */
  __construct__: function () {
    this.name = "ArgumentException";
    this.message = "An argument exception has occured";
    this.__paramName = null;
  },

  /**
   * @memberOf {util4js.ArgumentException}
   * @public
   * @property {String} paramName
   */
  paramName: {
    get: function () {
      return this.__paramName;
    },
    set: function (value) {
      return this.__paramName = value;
    } 
  },

  /**
   * @memberOf {util4js.ArgumentException}
   * @public
   * @overridden
   * @method toString
   * @retuns {String}
   */
  toString: function () {
    return this.name + ": " + this.message + " (Parameter: " + this.paramName  + ")";
  }

}, Exception);

exports.ArgumentException = ArgumentException;

/**
 * @class {util4js.ArgumentNullException}
 * @extends {util4js.ArgumentException}
 */
var ArgumentNullException = $class({
 
  /**
   * @memberOf {util4js.ArgumentNullException}
   * @public
   * @construct {util4js.ArgumentNullException}
   */
  __construct__: function () {
    this.name = "ArgumentNullException";
    this.message = "An argument null exception has occured";
  }

}, ArgumentException);

exports.ArgumentNullException = ArgumentNullException;

/**
 * @class {util4js.IndexOutOfRangeException}
 * @extends {util4js.Exception}
 */
var IndexOutOfRangeException = $class({

  /**
   * @memberOf {util4js.IndexOutOfRangeException}
   * @public
   * @construct {util4js.IndexOutOfRangeException}
   */
  __construct__: function () {
    this.name = "IndexOutOfRangeException";
    this.message = "Index out of range exception has occured";
  },

}, Exception);

exports.IndexOutOfRangeException = IndexOutOfRangeException;

/**
 * @interface {util4js.IIterator}
 */
var IIterator = $interface({

  /**
   * @memberOf {util4js.IIterator}
   * @method hasNext
   */
  hasNext: function () {
  },

  /**
   * @memberOf {util4js.IIterator}
   * @method next
   */
  next: function () {
  },

  /**
   * @memberOf {util4js.IIterator}
   * @method reset
   */
  reset: function () {
  }

});

exports.IIterator = IIterator;

/**
 * @interface {util4js.IIteratable}
 */
var IIteratable = $interface({

  /**
   * @memberOf {util4js.IIteratable}
   * @method iterator
   */
  iterator: function () {
  }

});

exports.IIteratable = IIteratable;

/**
 * @class {util4js.Iterator}
 * @extends {util4js.Object} 
 * @implements {util4js.IIterator}
 */
var Iterator = $class({

  /**
   * @memberOf {util4js.Iterator}
   * @public
   * @constructor {util4js.Iterator}
   * @param {Array} items
   */
  __construct__: function (items) {
    this.__index = -1;
    if (!items) {
      throw new ArgumentNullException({
        paramName: "items"
      });
    }
    if (items instanceof Array) {
      this.__items = items;
    } else {
      throw new ArgumentException({
        paramName: "items",
        message: "Parameter type is invalid"
      });
    }
  },

  /**
   * @memberOf {util4js.Iterator}
   * @public
   * @method hasNext 
   */
  hasNext: function () {
    return this.__index + 1 < this.__items.length;
  },

  /**
   * @memberOf {util4js.Iterator}
   * @public
   * @method next
   */
  next: function () {
    if (!this.hasNext()) {
      return null;
    }
    this.__index++;
    return this.__items[this.__index]; 
  },

  /**
   * @memberOf {util4js.Iterator}
   * @public
   * @method reset
   */
  reset: function () {
    this.__index = -1;
  },

  /**
   * @memberOf {util4js.Iterator}
   * @public
   * @method toString
   * @returns {String}
   */
  toString: function () {
    return "util4js.Iterator";
  }

}, Object, IIterator);

exports.Iterator = Iterator;

/**
 * @interface {util4js.IList}
 * @extends {util4js.IIteratable}
 */
var IList = $interface({

  /**
   * @memberOf {util4js.IList}
   * @property {Number} length
   */
  length: { 
    get: function () {
    }
  },

  /**
   * @memberOf {util4js.IList}
   * @method add
   * @param {Object} item
   * @returns {Number} index
   */
  add: function (item) {
  },

  /**
   * @memberOf {util4js.IList}
   * @method remove
   * @param {Object} item
   * @returns {Boolean}
   */
  remove: function (item) {
  },

  /**
   * @memberOf {util4js.IList}
   * @method clear
   */
  clear: function () {
  },

  /**
   * @memberOf {util4js.IList}
   * @method contains
   * @param {Object} item
   * @returns {Boolean}
   */
  contains: function (item) {
  },

  /**
   * @memberOf {util4js.IList}
   * @method get
   * @param {Number} index
   * @returns {Object}
   */
  get: function (index) {
  },

  /**
   * @memberOf {util4js.IList}
   * @method set
   * @param {Number} index
   * @param {Object} item
   */
  set: function (index, item) {
  },
 
  /**
   * @memberOf {util4js.IList}
   * @method insert
   * @param {Number} index
   * @param {Object} item
   */ 
  insert: function (index, item) {
  },

  /**
   * @memberOf {util4js.IList}
   * @method removeAt
   * @param {Number} index
   * @returns {Boolean}
   */
  removeAt: function (index) {
  },

  /**
   * @memberOf {util4js.IList}
   * @method indexOf
   * @param {Object} item
   * @returns {Number}
   */
  indexOf: function (item) { 
  } 

}, IIteratable);

exports.IList = IList;

/**
 * @class {util4js.List}
 * @extends {util4js.Object}
 * @implements {util4js.IList}
 */
var List = $class({

  /**
   * @memberOf {util4js.List}
   * @public
   * @constructor {util4js.List}
   * @param {Array} items
   */
  __construct__: function (items) {
    if (items) {
      if (items instanceof Array) {
        this.__items = items;
      } else {
        throw new ArgumentException({
          paramName: "items",
          message: "Parameter type is invalid"
        });
      } 
    } else {
      this.__items = [];
    }
  },

  /**
   * @memberOf {util4js.List}
   * @public
   * @method toString
   * @returns {String}
   */
  toString: function () {
    return "util4js.List";
  },

  /**
   * @memberOf {util4js.List}
   * @public
   * @method iterator
   * @returns {util4js.Iterator}
   */
  iterator: function () {
    return new Iterator(this.__items);
  },

  /**
   * @memberOf {util4js.List}
   * @public
   * @method count
   * @returns {Integer} length
   */
  length: {
    get: function () {
      return this.__items.length;
    }
  },

  /**
   * @memberOf {util4js.List}
   * @method remove
   * @param {Object} item
   * @returns {Boolean}
   */
  add: function (item) {
    this.__items.push(item);
  },

   /**
   * @memberOf {util4js.List}
   * @method remove
   * @param {Object} item
   * @returns {Boolean}
   */
  remove: function (item) {
    var index = this.__items.indexOf(item);
    if (index != -1) {
      this.__items.splice(index, 1); 
      return true;
    }
    return false;
  }, 

  /**
   * @memberOf {util4js.List}
   * @public
   * @method clear 
   */
  clear: function () {
    this.__items = [];
  },

  /**
   * @memberOf {util4js.List}
   * @public
   * @method contains
   */
  contains: function (item) {
    return this.__items.indexOf(item) != -1;
  },

  /**
   * @memberOf {util4js.List}
   * @method get
   * @param {Integer} index
   * @returns {Object}
   */
  get: function (index) {
    if (index < 0 || index >= this.count) {
      throw new IndexOutOfRangeException(); 
    } 
    return this.__items[index];
  },

  /**
   * @memberOf {util4js.List}
   * @method set
   * @param {Integer} index
   * @param {Object} item
   */
  set: function (index, item) {
    if (index < 0 || index >= this.count) {
      throw new IndexOutOfRangeException(); 
    } 
    this.__items[index] = item;
  },

  /**
   * @memberOf {util4js.List}
   * @method insert
   * @param {Integer} index
   * @param {Object} item
   */ 
  insert: function (index, item) {
    if (index < 0 || index > this.count) {
      throw new IndexOutOfRangeException(); 
    } 
    this.__items.splice(index, 0, item);
  },

    /**
   * @memberOf {util4js.List}
   * @method removeAt
   * @param {Integer} index
   * @returns {Boolean}
   */
  removeAt: function (index) {
    if (index < 0 || index >= this.count) {
      throw new IndexOutOfRangeException(); 
    } 
    this.__items.splice(index, 1);
    return true;
  },

  /**
   * @memberOf {util4js.List}
   * @method indexOf
   * @param {Object} item
   * @returns {Integer}
   */
  indexOf: function (item) {
    if ($is(item, Object)) {
      for (var i = 0; i < this.__items.length; i++) {
        if (Object.equals(this.__items[i], item)) {
          return i;
        }
      } 
      return -1;
    } else {
      return this.__items.indexOf(item); 
    }
  }

}, Object, IList);

exports.List = List;

/**
 * @interface {util4js.ISet}
 * @extends {util4js.ISet}
 */
var ISet = $interface({

  /**
   * @memberOf {util4js.ISet}
   * @property {Integer} length
   */
  length: { 
    get: function() {
    }
  },

  /**
   * @memberOf {util4js.ISet}
   * @method add
   * @param {Object} item
   * @returns {Integer} index
   */
  add: function (item) {
  },

  /**
   * @memberOf {util4js.ISet}
   * @method remove
   * @param {Object} item
   * @returns {Boolean}
   */
  remove: function (item) {
  },

  /**
   * @memberOf {util4js.ISet}
   * @method clear
   */
  clear: function () {
  },

  /**
   * @memberOf {util4js.ISet}
   * @method contains
   * @param {Object} item
   * @returns {Boolean}
   */
  contains: function (item) {
  }

}, ISet);

exports.ISet = ISet;

/**
 * @class {util4js.SetIterator}
 * @extends {util4js.Object}
 * @implements {util4js.IIterator}
 */
var SetIterator = $class({

  /**
   * @memberOf {util4js.SetIterator}
   * @public
   * @constructor {util4js.SetIterator}
   * @param {Object} items
   */
  __construct__: function (items) {
    this.__index = -1;
    if (!items) {
      throw new ArgumentNullException({
        paramName: "items"
      });
    }
    this.__items = items;
    this.__hashCodes = global.Object.keys(items);
  },

  /**
   * @memberOf {util4js.SetIterator}
   * @public
   * @method hasNext 
   */
  hasNext: function () {
    return this.__index + 1 < this.__hashCodes.length;
  },

  /**
   * @memberOf {util4js.SetIterator}
   * @public
   * @method next
   */
  next: function () {
    if (!this.hasNext()) {
      return null;
    }
    this.__index++;
    return this.__items[this.__hashCodes[this.__index]];
  },

  /**
   * @memberOf {util4js.SetIterator}
   * @public
   * @method reset
   */
  reset: function () {
    this.__index = -1;
  },

  /**
   * @memberOf {util4js.SetIterator}
   * @public
   * @method toString
   * @returns {String}
   */
  toString: function () {
    return "util4js.SetIterator";
  }

}, Object, IIterator);

exports.SetIterator = SetIterator;

/**
 * @class {util4js.Set}
 * @extends {util4js.Object}
 * @implements {util4js.ISet}
 */
var Set = $class({

  /**
   * @memberOf {util4js.Set}
   * @public
   * @construct {util4js.Set}
   * @param {Array} items
   */
  __construct__: function (items) {
    this.__length = 0;
    this.__items = global.Object.create(null);
    if (items) {
      if (items instanceof Array) {
        for (var i = 0; i< items.length; i++) {
          this.add(items[i]);
        }  
      } else {
        throw new ArgumentException({
          paramName: "items",
          message: "Parameter type is invalid"
        }); 
      }
    }
  },

  /**
   * @memberOf {util4js.Set}
   * @public
   * @method toString
   * @returns {String}
   */
  toString: function () {
    return "util4js.Set";
  },

  /**
   * @memberOf {util4js.Set}
   * @public
   * @method iterator
   * @returns {util4js.Iterator}
   */
  iterator: function () {
    return new SetIterator(this.__items);
  },

  /**
   * @memberOf {util4js.Set}
   * @public
   * @property {Number}
   */
  length: { 
    get: function() {
      return this.__length;
    }
  },

  /**
   * @memberOf {util4js.Set}
   * @public
   * @method add
   * @param {Object} item
   * @returns {Integer} index
   */
  add: function (item) {
    var hashCode = this.__getItemHashCode(item);
    if (!this.contains(hashCode)) {
      this.__items[hashCode] = item;
      this.__length++;
    } else {
      throw new ArgumentException({
        paramName: "item",
        message: "A dublicated item can't be added to the set"
      }); 
    }
  },

  /**
   * @memberOf {util4js.Set}
   * @public
   * @method remove
   * @param {Object} item
   * @returns {Boolean}
   */
  remove: function (item) {
    var hashCode = this.__getItemHashCode(item);
    if (this.contains(hashCode)) {
      delete this.__items[hashCode];
      this.__length--;
      return true;
    }
    return false;
  },

  /**
   * @memberOf {util4js.Set}
   * @public
   * @method clear
   */
  clear: function () {
    this.__length = 0;
    this.__items = {}; 
  },

  /**
   * @memberOf {util4js.Set}
   * @public
   * @method contains
   * @param {Object} item
   * @returns {Boolean}
   */
  contains: function (item) {
    var hashCode = this.__getItemHashCode(item);
    return (hashCode in this.__items); 
  },

  /**
   * @memberOf {util4js.Set}
   * @private
   * @method __getItemHashCode
   * @param {Object} item
   * @returns {Number}
   */
  __getItemHashCode: function (item) {
    if (!item) {
      throw new ArgumentNullException({
        paramName: "item"
      });
    }
    if ($is(item, Object)) {
      return item.getHashCode();
    } else if (typeof item === "object") {
      throw new ArgumentException({
        paramName: "item",
        message: "Type is invalid"
      }); 
    }
    return item;
  }

}, Object, ISet);

exports.Set = Set;

/**
 * @interface {util4js.IDictionary}
 * @extends {util4js.IIteratable}
 */
var IDictionary = $interface({

  /**
   * @memberOf {util4js.IDictionary}
   * @property {Number} length
   */
  length: { 
    get: function() {
    }
  },

  /**
   * @memberOf {util4js.IDictionary}
   * @method clear
   */
  clear: function () {
  },

  /**
   * @memberOf {util4js.IDictionary}
   * @method add
   * @param {Object} key
   * @param {Object} value
   */
  add: function (key, value) {
  },
   
  /**
   * @memberOf {util4js.IDictionary}
   * @method remove
   * @param {Object} key
   */ 
  remove: function (key) {
  },

  /**
   * @memberOf {util4js.IDictionary}
   * @method contains
   * @param {Object} key
   * @returns {Boolean}
   */
  contains: function (key) {
  },

  /**
   * @memberOf {util4js.IDictionary}
   * @method get
   * @param {Object} key
   * @returns {Boolean}
   */
  get: function (key) {
  },

  /**
   * @meberOf {util4js.IDictionary}
   * @method set
   * @param {Object} key
   * @param {Object} value
   */
  set: function (key, value) {
  }

}, IIteratable);

exports.IDictionary = IDictionary;

/**
 * @class {util4js.KeyValuePair}
 * @extends {util4js.Object}
 */
var KeyValuePair = $class({

  /**
   * @memberOf {util4js.KeyValuePair}
   * @public
   * @constructor {util4js.KeyValuePair}
   * @param {Object} key
   * @param {Object} value
   */
  __construct__: function (key, value) {
    if (!key) {
      throw new ArgumentNullException("Key is not set");
    }
    this.__key = key;
    this.__value = value;
  },

  /**
   * @memberOf {util4js.KeyValuePair}
   * @public
   * @property {Object} key
   */
  key: {
    get: function () {
      return this.__key;
    }
  },

  /**
   * @memberOf {util4js.KeyValuePair}
   * @public
   * @property {Object} value
   */
  value: {
    get: function () {
      return this.__value;
    },
    set: function (value) {
      this.__value = value;
    }
  }

}, Object);

exports.KeyValuePair = KeyValuePair;

/**
 * @class {util4js.DictionaryIterator}
 * @extends {util4js.Object}
 * @implements {util4js.IIterator}
 */
var DictionaryIterator = $class({

  /**
   * @memberOf {util4js.DictionaryIterator}
   * @public
   * @constructor {util4js.DictionaryIterator}
   * @param {Object} items
   */
  __construct__: function (items) {
    this.__index = -1; 
    if (!items) {
      throw new ArgumentNullException({
        paramName: "items"
      });
    }
    this.__items = items;
    this.__hashCodes = global.Object.keys(items);
  },

  /**
   * @memberOf {util4js.DictionaryIterator}
   * @public
   * @method hasNext 
   */
  hasNext: function () {
    return this.__index + 1 < this.__hashCodes.length;
  },

  /**
   * @memberOf {util4js.DictionaryIterator}
   * @public
   * @method next
   */
  next: function () {
    if (!this.hasNext()) {
      return null;
    }
    this.__index++;
    return this.__items[this.__hashCodes[this.__index]];
  },

  /**
   * @memberOf {util4js.DictionaryIterator}
   * @public
   * @method reset
   */
  reset: function () {
    this.__index = -1;
  },

  /**
   * @memberOf {util4js.DictionaryIterator}
   * @public
   * @method toString
   * @returns {String}
   */
  toString: function () {
    return "util4js.DictionaryIterator";
  }

}, Object, IIterator);

exports.DictionaryIterator = DictionaryIterator;

/**
 * @class {util4js.Dictionary}
 * @extends {util4js.Object}
 * @implements {util4js.IDictionary}
 */
var Dictionary = $class({

  /**
   * @memberOf {util4js.Dictionary}
   * @public
   * @constructor {util4js.Dictionary}
   * @param {Array} items
   */
  __construct__: function (items) {
    this.__items = global.Object.create(null);
    this.__length = 0;
    if (items) {
      if (items instanceof Array) {
        for (var i = 0; i < items.length; i++) {
          if ("key" in items[i] && "value" in items[i]) {
            this.add(items[i].key, items[i].value);
          } else {
            throw new ArgumentException({
              paramName: "items"
            }); 
          }
        }
      } else {
        throw new ArgumentException({
          paramName: "items",
          message: "Parameter type is invalid"
        });
      }
    }
  },

  /**
   * @memberOf {util4js.Dictionary}
   * @public
   * @method toString
   * @returns {String}
   */
  toString: function () {
    return "util4js.Dictionary";
  },

  /**
   * @memberOf {util4js.Dictionary}
   * @public
   * @method iterator
   * @returns {util4js.Iterator}
   */
  iterator: function () {
    return new DictionaryIterator(this.__items);
  },

  /**
   * @memberOf {util4js.Dictionary}
   * @public
   * @property {Number} length
   */
  length: {
    get: function () {
      return this.__length;
    }
  },

  /**
   * @memberOf {util4js.Dictionary}
   * @public
   * @methos clear
   */
  clear: function () {
    this.__length = 0;
    this.__items = {};
  },

  /**
   * @memberOf {util4js.Dictionary}
   * @public
   * @method add
   * @param {Object} key
   * @param {Object} value
   */
  add: function (key, value) {
    var hashCode = this.__getKeyHashCode(key);
    if (!this.contains(hashCode)) {
      this.__items[hashCode] = new KeyValuePair(key, value);
      this.__length++;
    } else {
      throw new ArgumentException({ 
        paramName: "key",
        message: "An entry with dublicated key can't be added to the dictionary"
      });
    }
  },

  /**
   * @memberOf {util4js.Dictionary}
   * @public
   * @method remove
   * @param {Object} key
   * @returns {Boolean}
   */
  remove: function (key) {
    var hashCode = this.__getKeyHashCode(key);
    if (this.contains(hashCode)) {
      delete this.__items[hashCode];
      this.__length--;
      return true;
    }
    return false;
  },

  /**
   * @memberOf {util4js.Dictionary}
   * @public
   * @method contains
   * @param {Object} key
   * @returns {Boolean}
   */
  contains: function (key) {
    var hashCode = this.__getKeyHashCode(key);
    return (hashCode in this.__items);
  },

  /**
   * @memberOf {util4js.Dictionary}
   * @public
   * @method get
   * @param {Object} key
   * @returns {Object}
   */
  get: function (key) {
    var hashCode = this.__getKeyHashCode(key);
    if (this.contains(hashCode)) {
      return this.__items[hashCode].value;
    } else {
      throw new ArgumentException({
        paramName: "key",
        message: "Key doesn't exists in the dictionary"
      });
    }
  },

  /**
   * @memberOf {util4js.Dictionary}
   * @public
   * @method set
   * @param {Object} key
   * @param {Object} value
   */
  set: function (key, value) {
    var hashCode = this.__getKeyHashCode(key);
    if (this.contains(hashCode)) {
      this.__items[hashCode].value = value;
    } else {
      throw new ArgumentException({ 
        paramName: "key",
        message: "Key doesn't exists in the dictionary" 
      });
    }
  },

  /**
   * @memberOf {util4js.Dictionary}
   * @private
   * @method __getKeyHashCode
   * @param {Object} key
   */
  __getKeyHashCode: function (key) {
    if (!key) {
      throw new ArgumentNullException({ 
        paramName: "key"
      });
    }
    if ($is(key, Object)) {
      return key.getHashCode();
    } else if (typeof key === "object") {
      throw new ArgumentException({
        paramName: "key",
        message: "Type is invalid"
      });
    }
    return key;
  }

}, Object, IDictionary);

exports.Dictionary = Dictionary;

/**
 * @interface {util4js.IStack}
 * @extends {util4js.IIteratable}
 */
var IStack = $interface({

  /**
   * @memberOf {util4js.IStack}
   * @property {Number} length
   */
  length: {
    get: function () {
    }
  },

  /**
   * @memberOf {util4js.IStack}
   * @method clear
   */
  clear: function () {
  },

  /**
   * @memberOf {util4js.IStack}
   * @method contains
   * @param {Object} item
   */
  contains: function (item) {
  },

  /**
   * @memberOf {util4js.IStack}
   * @method peek
   * @returns {Object}
   */
  peek: function () {
  },

  /**
   * @memberOf {util4js.IStack}
   * @method pop
   * @returns {Object}
   */
  pop: function () {
  },

  /**
   * @memberOf {util4js.IStack}
   * @method push
   * @param {Object} item
   */
  push: function (item) {
  }

}, IIteratable);

exports.IStack = IStack;

/**
 * @class {util4js.Stack}
 * @extends {util4js.Object}
 * @implements {util4js.IStack}
 */
var Stack = $class({

  /**
   * @memberOf {util4js.Stack}
   * @public
   * @constructor {util4js.Stack}
   * @param {Array} items
   */
  __construct__: function (items) {
    this.__items = [];
    if (items) {
      if (items instanceof Array) {
        for (var i = 0; i < items.length; i++) {
          this.push(items[i]);
        }  
      } else {
        throw new ArgumentException({
          paramName: "items",
          message: "Parameter type is invalid"
        });
      } 
    }
  },

  /**
   * @memberOf {util4js.Stack}
   * @public
   * @method toString
   * @returns {String}
   */
  toString: function () {
    return "util4js.Stack";
  },

  /**
   * @memberOf {util4js.Stack}
   * @public
   * @method iterator
   * @returns {util4js.Iterator}
   */
  iterator: function () {
    return new Iterator(this.__items);
  },

  /**
   * @memberOf {util4js.Stack}
   * @public
   * @property {Number} length
   */
  length: {
    get: function () {
      return this.__items.length;
    }
  },

  /**
   * @memberOf {util4js.Stack}
   * @public
   * @method clear
   */
  clear: function () {
    this.__items = [];
  },

  /**
   * @memberOf {util4js.Stack}
   * @public
   * @method contains
   * @param {Object} item
   */
  contains: function (item) {
    var index;
    if ($is(item, Object)) {
      for (var i = 0; i < this.__items.length; i++) {
        if (Object.equals(this.__items[i], item)) {
          index = i;
          break;
        }
      } 
    } else {
      index = this.__items.indexOf(item);
    }
    return index != -1;
  },

  /**
   * @memberOf {util4js.Stack}
   * @public
   * @method peek
   * @returns {Object}
   */
  peek: function () {
    if (this.__items.length == 0) {
      throw new Exception({
        message: "Stack is empty"
      });
    }
    return this.__items[this.__items.length - 1];
  },

  /**
   * @memberOf {util4js.Stack}
   * @public
   * @method pop
   * @returns {Object}
   */
  pop: function () {
    if (this.__items.length == 0) {
      throw new Exception({
        message: "Stack is empty"
      });
    }
    return this.__items.pop();
  },

  /**
   * @memberOf {util4js.Stack}
   * @public
   * @method push
   * @param {Object} item
   */
  push: function (item) {
    this.__items.push(item);
  }

}, Object, IStack);

exports.Stack = Stack;

/**
 * @interface {util4js.IQueue}
 * @extends {util4js.IIteratable}
 */
var IQueue = $interface({

  /**
   * @memberOf {util4js.IQueue}
   * @property {Number} length
   */
  length: {
    get: function () {
    }
  },

  /**
   * @memberOf {util4js.IQueue}
   * @public
   * @method clear
   */
  clear: function () {
    
  },

  /**
   * @memberOf {util4js.IQueue}
   * @public
   * @method contains
   * @param {Object} item
   */
  contains: function (item) {
  },

  /**
   * @memberOf {util4js.IQueue}
   * @public
   * @method peek
   * @returns {Object}
   */
  peek: function () {
  },

  /**
   * @memberOf {util4js.IQueue}
   * @public
   * @method enqueue
   * @param {Object} item
   */
  enqueue: function (item) {
  },

  /**
   * @memberOf {util4js.IQueue}
   * @public
   * @method dequeue
   * @returns {Object}
   */
  dequeue: function () { 
  }

});

exports.IQueue = IQueue;

/**
 * @class {util4js.Queue}
 * @extends {util4js.Object}
 * @implements {util4js.IQueque}
 */
var Queue = $class({

  /**
   * @memberOf {util4js.Queue}
   * @public
   * @constructor {util4js.Queue}
   * @param {Array} items
   */
  __construct__: function (items) {
    this.__items = [];
    if (items) {
      if (items instanceof Array) {
        for (var i =0; i< items.length; i++) {
          this.enqueue(items[i]);
        } 
      } else {
        throw new ArgumentException({
          paramName: "items",
          message: "Parameter type is invalid"
        });
      } 
    }
  },

  /**
   * @memberOf {util4js.Queue}
   * @public
   * @method toString
   * @returns {String}
   */
  toString: function () {
    return "util4js.Queue";
  },

  /**
   * @memberOf {util4js.Queue}
   * @public
   * @method iterator
   * @returns {util4js.Iterator}
   */
  iterator: function () {
    return new Iterator(this.__items);
  },

  /**
   * @memberOf {util4js.Queue}
   * @public
   * @property {Number} length
   */
  length: {
    get: function () {
      return this.__items.length;
    }
  },

  /**
   * @memberOf {util4js.Queue}
   * @public
   * @method clear
   */
  clear: function () {
    this.__items = [];
  },

  /**
   * @memberOf {util4js.Queue}
   * @public
   * @method contains
   * @param {Object} item
   */
  contains: function (item) {
    var index;
    if ($is(item, Object)) {
      for (var i = 0; i < this.__items.length; i++) {
        if (Object.equals(this.__items[i], item)) {
          index = i;
          break;
        }
      } 
    } else {
      index = this.__items.indexOf(item);
    }
    return index != -1;
  },

  /**
   * @memberOf {util4js.Queue}
   * @public
   * @method peek
   * @returns {Object}
   */
  peek: function () {
    if (this.__items.length == 0) {
      throw new Exception({
        message: "Queue is empty"
      });
    }
    return this.__items[0]; 
  },

  /**
   * @memberOf {util4js.Queue}
   * @public
   * @method enqueue
   * @param {Object} item
   */
  enqueue: function (item) {
    this.__items.push(item);
  },

  /**
   * @memberOf {util4js.Queue}
   * @public
   * @method dequeue
   * @returns {Object}
   */
  dequeue: function () {
    if (this.__items.length == 0) {
      throw new Exception({
        message: "Queue is empty"
      });
    }
    return this.__items.shift();
  }

}, Object, IQueue);

exports.Queue = Queue;

/**
 * @extension {util4js.IIteratable}
 * @method forEach
 * @param {Function} callback
 */
$extend(IIteratable, "forEach", function (callback) {
  if (callback) {
    var iterator = this.iterator();
    while (iterator.hasNext()) {
      callback(iterator.next());
    } 
  } 
});

/**
 * @extension {util4js.IIteratable}
 * @method count
 * @returns {Number}
 */
$extend(IIteratable, "count", function () {
  var count = 0;
  var iterator = this.iterator();
  while (iterator.next()) {
    count++;  
  }
  return count;
});

/**
 * @extension {util4js.IIteratable}
 * @method first
 * @returns {Object}
 */
$extend(IIteratable, "first", function () {
  var iterator = this.iterator();
  var first = null;
  while (iterator.hasNext()) {
    first = iterator.next();
    break;
  }
  return first;
});

/**
 * @extension {util4js.IIteratable}
 * @method last
 * @returns {Object}
 */
$extend(IIteratable, "last", function () {
  var iterator = this.iterator();
  var last = null;
  while (iterator.hasNext()) {
    last = iterator.next();
  }
  return last;
});

/**
 * @extension {util4js.IIteratable}
 * @method toArray
 * @returns {Object[]}
 */
$extend(IIteratable, "toArray", function () {
  var array = [];
  var iterator = this.iterator();  
  while (iterator.hasNext()) {
    array.push(iterator.next());
  }
  return array;
});

/**
 * @extension {util4js.IIteratable}
 * @method toList
 * @returns {util4js.IList}
 */
$extend(IIteratable, "toList", function () {
  var list = new List();
  while (iterator.hasNext()) {
    list.add(iterator.next());
  }
  return list; 
});

/**
 * @class {util4js.Event}
 * @extends {util4js.Object}
 */
var Event = $class({

  /**
   * @memberOf {util4js.Event}
   * @public
   * @constructor {util4js.Event}
   */
  __construct__: function () {
    this.__target = null;
    this.__type = "";
  },

  /**
   * @memberOf {util4js.Event}
   * @public
   * @property {String} type
   */
  type: {
    get: function () {
      return this.__type;
    }, 
    set: function (value) {
      this.__type = value;
    }  
  },

  /**
   * @memberOf {util4js.Event}
   * @public
   * @property {Object} target
   */
  target: {
    get: function () {
      return this.__target;
    },
    set: function (value) {
      this.__target = value;
    }
  },

  /**
   * @memberOf {util4js.Event}
   * @public
   * @overridden
   * @method toString
   * @returns {String}
   */
  toString: function () {
    return this.type;
  }

}, Object);

exports.Event = Event;

/**
 * @interface {util4js.IEventTarget}
 */
var IEventTarget = $interface({

  /**
   * @memberOf {util4js.IEventTarget}
   * @method addEventListener
   * @param {String} type
   * @param {Function} listener
   */
  addEventListener: function (type, listener) { 
  },

  /**
   * @memberOf {util4js.IEventTarget}
   * @method removeEventListener
   * @param {String} type
   * @param {Function} listener
   */
  removeEventListener: function (type, listener) { 
  },

  /**
   * @memberOf {util4js.IEventTarget}
   * @method dispatchEvent
   * @param {util4js.Event} e 
   */ 
  dispatchEvent: function (e) { 
  }

});

exports.IEventTarget = IEventTarget;

/**
 * @class {util4js.EventDispathcer}
 * @extends {util4js.Object}
 */
var EventDispathcer = $class({

  /**
   * @memberOf {util4js.EventDispathcer}
   * @public
   * @constructor {util4js.EventDispathcer}
   */
  __construct__: function () {
    this.__listeners = new Dictionary();
  },

  /**
   * @memberOf {util4js.EventDispathcer}
   * @public
   * @method add
   * @param {String} type
   * @param {Function} listener
   */
  add: function (type, listener) {
    if (!type) {
      throw new ArgumentNullException({
        paramName: "type"
      });
    }
    if (typeof type !== "string") {
      throw new ArgumentException({
        paramName: "type",
        message: "Type is invalid"
      });
    }
    if (!listener) {
      throw new ArgumentNullException({
        paramName: "listener"
      });
    }
    if (typeof listener !== "function") {
      throw new ArgumentException({
        paramName: "listener",
        message: "Type is invalid"
      });
    }
    if (this.__listeners.contains(type)) {
      this.__listeners.get(type).add(listener);
    } else {
      this.__listeners.add(type, new List([listener]));
    }
  },

  /**
   * @memberOf {util4js.EventDispathcer}
   * @public
   * @method remove
   * @param {String} type
   * @param {Function} listener
   * @returns {Boolean}
   */
  remove: function (type, listener) {
    if (!type) {
      throw new ArgumentNullException({
        paramName: "type"
      });
    }
    if (typeof type !== "string") {
      throw new ArgumentException({
        paramName: "type",
        message: "Type is invalid"
      });
    }
    if (!listener) {
      throw new ArgumentNullException({
        paramName: "listener"
      });
    }
    if (typeof listener !== "function") {
      throw new ArgumentException({
        paramName: "listener",
        message: "Type is invalid"
      });
    }
    return this.__listeners.get(type).remove(listener);
  },

  /**
   * @memberOf {util4js.EventDispathcer}
   * @public
   * @method dispatch
   * @param {Event} e
   */
  dispatch: function (e) {
    if (!e) {
      throw new ArgumentNullException({
        paramName: "e"
      });
    }
    if ($is(e, Event)) {
      this.__listeners.get(e.type).forEach(function (callback) {
        callback.call(e.target, e);    
      });  
    } else {
      throw new ArgumentException({
        paramName: "e",
        message: "Parameter type is invalid"
      });  
    }
  },

  /**
   * @memberOf {util4js.EventDispathcer}
   * @public
   * @overridden
   * @method toString
   * @retuns {String}
   */
  toString: function () {
    return "util4js.EventDispathcer"; 
  }

}, Object);

exports.EventDispathcer = EventDispathcer;

/**
 * @abstract
 * @class {util4js.Component}
 * @extends {util4js.Object}
 */
var Component = $abstract_class({

  /**
   * @memberOf {util4js.Component}
   * @public
   * @constructor {util4js.Component}
   */
  __construct__: function () {
    this.__eventDispatcher = new EventDispathcer();
  },

  /**
   * @memberOf {util4js.Component}
   * @public
   * @method addEventListener
   * @param {String} type
   * @param {Function} listener
   */
  addEventListener: function (type, listener) { 
    this.__eventDispatcher.add(type, listener);
  },

  /**
   * @memberOf {util4js.Component}
   * @public
   * @method removeEventListener
   * @param {String} type
   * @param {Function} listener
   */
  removeEventListener: function (type, listener) { 
    this.__eventDispatcher.remove(type, listener);
  },

  /**
   * @memberOf {util4js.Component}
   * @public
   * @method dispatchEvent
   * @param {util4js.Event} e 
   */ 
  dispatchEvent: function (e) { 
    this.__eventDispatcher.dispatch($create(Event, e));
  },

  /**
   * @memberOf {util4js.Component}
   * @public
   * @overridden
   * @method toString
   * @returns {String}
   */
  toString: function () {
    return "util4js.Component";
  }

}, Object, IEventTarget);

exports.Component = Component;

