/**
 * @class {util4js.DictionaryIterator}
 * @extends {util4js.Object}
 * @implements {util4js.IIterator}
 */
var DictionaryIterator = $class({

  /**
   * @memberOf {util4js.DictionaryIterator}
   * @public
   * @constructor {util4js.DictionaryIterator}
   * @param {Object} items
   */
  __construct__: function (items) {
    this.__index = -1; 
    if (!items) {
      throw new ArgumentNullException({
        paramName: "items"
      });
    }
    this.__items = items;
    this.__hashCodes = global.Object.keys(items);
  },

  /**
   * @memberOf {util4js.DictionaryIterator}
   * @public
   * @method hasNext 
   */
  hasNext: function () {
    return this.__index + 1 < this.__hashCodes.length;
  },

  /**
   * @memberOf {util4js.DictionaryIterator}
   * @public
   * @method next
   */
  next: function () {
    if (!this.hasNext()) {
      return null;
    }
    this.__index++;
    return this.__items[this.__hashCodes[this.__index]];
  },

  /**
   * @memberOf {util4js.DictionaryIterator}
   * @public
   * @method reset
   */
  reset: function () {
    this.__index = -1;
  },

  /**
   * @memberOf {util4js.DictionaryIterator}
   * @public
   * @method toString
   * @returns {String}
   */
  toString: function () {
    return "util4js.DictionaryIterator";
  }

}, Object, IIterator);

exports.DictionaryIterator = DictionaryIterator;

