/**
 * @class {util4js.EventDispathcer}
 * @extends {util4js.Object}
 */
var EventDispathcer = $class({

  /**
   * @memberOf {util4js.EventDispathcer}
   * @public
   * @constructor {util4js.EventDispathcer}
   */
  __construct__: function () {
    this.__listeners = new Dictionary();
  },

  /**
   * @memberOf {util4js.EventDispathcer}
   * @public
   * @method add
   * @param {String} type
   * @param {Function} listener
   */
  add: function (type, listener) {
    if (!type) {
      throw new ArgumentNullException({
        paramName: "type"
      });
    }
    if (typeof type !== "string") {
      throw new ArgumentException({
        paramName: "type",
        message: "Type is invalid"
      });
    }
    if (!listener) {
      throw new ArgumentNullException({
        paramName: "listener"
      });
    }
    if (typeof listener !== "function") {
      throw new ArgumentException({
        paramName: "listener",
        message: "Type is invalid"
      });
    }
    if (this.__listeners.contains(type)) {
      this.__listeners.get(type).add(listener);
    } else {
      this.__listeners.add(type, new List([listener]));
    }
  },

  /**
   * @memberOf {util4js.EventDispathcer}
   * @public
   * @method remove
   * @param {String} type
   * @param {Function} listener
   * @returns {Boolean}
   */
  remove: function (type, listener) {
    if (!type) {
      throw new ArgumentNullException({
        paramName: "type"
      });
    }
    if (typeof type !== "string") {
      throw new ArgumentException({
        paramName: "type",
        message: "Type is invalid"
      });
    }
    if (!listener) {
      throw new ArgumentNullException({
        paramName: "listener"
      });
    }
    if (typeof listener !== "function") {
      throw new ArgumentException({
        paramName: "listener",
        message: "Type is invalid"
      });
    }
    return this.__listeners.get(type).remove(listener);
  },

  /**
   * @memberOf {util4js.EventDispathcer}
   * @public
   * @method dispatch
   * @param {Event} e
   */
  dispatch: function (e) {
    if (!e) {
      throw new ArgumentNullException({
        paramName: "e"
      });
    }
    if ($is(e, Event)) {
      this.__listeners.get(e.type).forEach(function (callback) {
        callback.call(e.target, e);    
      });  
    } else {
      throw new ArgumentException({
        paramName: "e",
        message: "Parameter type is invalid"
      });  
    }
  },

  /**
   * @memberOf {util4js.EventDispathcer}
   * @public
   * @overridden
   * @method toString
   * @retuns {String}
   */
  toString: function () {
    return "util4js.EventDispathcer"; 
  }

}, Object);

exports.EventDispathcer = EventDispathcer;

