/**
 * @class {util4js.Iterator}
 * @extends {util4js.Object} 
 * @implements {util4js.IIterator}
 */
var Iterator = $class({

  /**
   * @memberOf {util4js.Iterator}
   * @public
   * @constructor {util4js.Iterator}
   * @param {Array} items
   */
  __construct__: function (items) {
    this.__index = -1;
    if (!items) {
      throw new ArgumentNullException({
        paramName: "items"
      });
    }
    if (items instanceof Array) {
      this.__items = items;
    } else {
      throw new ArgumentException({
        paramName: "items",
        message: "Parameter type is invalid"
      });
    }
  },

  /**
   * @memberOf {util4js.Iterator}
   * @public
   * @method hasNext 
   */
  hasNext: function () {
    return this.__index + 1 < this.__items.length;
  },

  /**
   * @memberOf {util4js.Iterator}
   * @public
   * @method next
   */
  next: function () {
    if (!this.hasNext()) {
      return null;
    }
    this.__index++;
    return this.__items[this.__index]; 
  },

  /**
   * @memberOf {util4js.Iterator}
   * @public
   * @method reset
   */
  reset: function () {
    this.__index = -1;
  },

  /**
   * @memberOf {util4js.Iterator}
   * @public
   * @method toString
   * @returns {String}
   */
  toString: function () {
    return "util4js.Iterator";
  }

}, Object, IIterator);

exports.Iterator = Iterator;

