/**
 * @class {util4js.List}
 * @extends {util4js.Object}
 * @implements {util4js.IList}
 */
var List = $class({

  /**
   * @memberOf {util4js.List}
   * @public
   * @constructor {util4js.List}
   * @param {Array} items
   */
  __construct__: function (items) {
    if (items) {
      if (items instanceof Array) {
        this.__items = items;
      } else {
        throw new ArgumentException({
          paramName: "items",
          message: "Parameter type is invalid"
        });
      } 
    } else {
      this.__items = [];
    }
  },

  /**
   * @memberOf {util4js.List}
   * @public
   * @method toString
   * @returns {String}
   */
  toString: function () {
    return "util4js.List";
  },

  /**
   * @memberOf {util4js.List}
   * @public
   * @method iterator
   * @returns {util4js.Iterator}
   */
  iterator: function () {
    return new Iterator(this.__items);
  },

  /**
   * @memberOf {util4js.List}
   * @public
   * @method count
   * @returns {Integer} length
   */
  length: {
    get: function () {
      return this.__items.length;
    }
  },

  /**
   * @memberOf {util4js.List}
   * @method remove
   * @param {Object} item
   * @returns {Boolean}
   */
  add: function (item) {
    this.__items.push(item);
  },

   /**
   * @memberOf {util4js.List}
   * @method remove
   * @param {Object} item
   * @returns {Boolean}
   */
  remove: function (item) {
    var index = this.__items.indexOf(item);
    if (index != -1) {
      this.__items.splice(index, 1); 
      return true;
    }
    return false;
  }, 

  /**
   * @memberOf {util4js.List}
   * @public
   * @method clear 
   */
  clear: function () {
    this.__items = [];
  },

  /**
   * @memberOf {util4js.List}
   * @public
   * @method contains
   */
  contains: function (item) {
    return this.__items.indexOf(item) != -1;
  },

  /**
   * @memberOf {util4js.List}
   * @method get
   * @param {Integer} index
   * @returns {Object}
   */
  get: function (index) {
    if (index < 0 || index >= this.count) {
      throw new IndexOutOfRangeException(); 
    } 
    return this.__items[index];
  },

  /**
   * @memberOf {util4js.List}
   * @method set
   * @param {Integer} index
   * @param {Object} item
   */
  set: function (index, item) {
    if (index < 0 || index >= this.count) {
      throw new IndexOutOfRangeException(); 
    } 
    this.__items[index] = item;
  },

  /**
   * @memberOf {util4js.List}
   * @method insert
   * @param {Integer} index
   * @param {Object} item
   */ 
  insert: function (index, item) {
    if (index < 0 || index > this.count) {
      throw new IndexOutOfRangeException(); 
    } 
    this.__items.splice(index, 0, item);
  },

    /**
   * @memberOf {util4js.List}
   * @method removeAt
   * @param {Integer} index
   * @returns {Boolean}
   */
  removeAt: function (index) {
    if (index < 0 || index >= this.count) {
      throw new IndexOutOfRangeException(); 
    } 
    this.__items.splice(index, 1);
    return true;
  },

  /**
   * @memberOf {util4js.List}
   * @method indexOf
   * @param {Object} item
   * @returns {Integer}
   */
  indexOf: function (item) {
    if ($is(item, Object)) {
      for (var i = 0; i < this.__items.length; i++) {
        if (Object.equals(this.__items[i], item)) {
          return i;
        }
      } 
      return -1;
    } else {
      return this.__items.indexOf(item); 
    }
  }

}, Object, IList);

exports.List = List;

