/**
 * @class {util4js.Queue}
 * @extends {util4js.Object}
 * @implements {util4js.IQueque}
 */
var Queue = $class({

  /**
   * @memberOf {util4js.Queue}
   * @public
   * @constructor {util4js.Queue}
   * @param {Array} items
   */
  __construct__: function (items) {
    this.__items = [];
    if (items) {
      if (items instanceof Array) {
        for (var i =0; i< items.length; i++) {
          this.enqueue(items[i]);
        } 
      } else {
        throw new ArgumentException({
          paramName: "items",
          message: "Parameter type is invalid"
        });
      } 
    }
  },

  /**
   * @memberOf {util4js.Queue}
   * @public
   * @method toString
   * @returns {String}
   */
  toString: function () {
    return "util4js.Queue";
  },

  /**
   * @memberOf {util4js.Queue}
   * @public
   * @method iterator
   * @returns {util4js.Iterator}
   */
  iterator: function () {
    return new Iterator(this.__items);
  },

  /**
   * @memberOf {util4js.Queue}
   * @public
   * @property {Number} length
   */
  length: {
    get: function () {
      return this.__items.length;
    }
  },

  /**
   * @memberOf {util4js.Queue}
   * @public
   * @method clear
   */
  clear: function () {
    this.__items = [];
  },

  /**
   * @memberOf {util4js.Queue}
   * @public
   * @method contains
   * @param {Object} item
   */
  contains: function (item) {
    var index;
    if ($is(item, Object)) {
      for (var i = 0; i < this.__items.length; i++) {
        if (Object.equals(this.__items[i], item)) {
          index = i;
          break;
        }
      } 
    } else {
      index = this.__items.indexOf(item);
    }
    return index != -1;
  },

  /**
   * @memberOf {util4js.Queue}
   * @public
   * @method peek
   * @returns {Object}
   */
  peek: function () {
    if (this.__items.length == 0) {
      throw new Exception({
        message: "Queue is empty"
      });
    }
    return this.__items[0]; 
  },

  /**
   * @memberOf {util4js.Queue}
   * @public
   * @method enqueue
   * @param {Object} item
   */
  enqueue: function (item) {
    this.__items.push(item);
  },

  /**
   * @memberOf {util4js.Queue}
   * @public
   * @method dequeue
   * @returns {Object}
   */
  dequeue: function () {
    if (this.__items.length == 0) {
      throw new Exception({
        message: "Queue is empty"
      });
    }
    return this.__items.shift();
  }

}, Object, IQueue);

exports.Queue = Queue;

