/**
 * @class {util4js.Set}
 * @extends {util4js.Object}
 * @implements {util4js.ISet}
 */
var Set = $class({

  /**
   * @memberOf {util4js.Set}
   * @public
   * @construct {util4js.Set}
   * @param {Array} items
   */
  __construct__: function (items) {
    this.__length = 0;
    this.__items = global.Object.create(null);
    if (items) {
      if (items instanceof Array) {
        for (var i = 0; i< items.length; i++) {
          this.add(items[i]);
        }  
      } else {
        throw new ArgumentException({
          paramName: "items",
          message: "Parameter type is invalid"
        }); 
      }
    }
  },

  /**
   * @memberOf {util4js.Set}
   * @public
   * @method toString
   * @returns {String}
   */
  toString: function () {
    return "util4js.Set";
  },

  /**
   * @memberOf {util4js.Set}
   * @public
   * @method iterator
   * @returns {util4js.Iterator}
   */
  iterator: function () {
    return new SetIterator(this.__items);
  },

  /**
   * @memberOf {util4js.Set}
   * @public
   * @property {Number}
   */
  length: { 
    get: function() {
      return this.__length;
    }
  },

  /**
   * @memberOf {util4js.Set}
   * @public
   * @method add
   * @param {Object} item
   * @returns {Integer} index
   */
  add: function (item) {
    var hashCode = this.__getItemHashCode(item);
    if (!this.contains(hashCode)) {
      this.__items[hashCode] = item;
      this.__length++;
    } else {
      throw new ArgumentException({
        paramName: "item",
        message: "A dublicated item can't be added to the set"
      }); 
    }
  },

  /**
   * @memberOf {util4js.Set}
   * @public
   * @method remove
   * @param {Object} item
   * @returns {Boolean}
   */
  remove: function (item) {
    var hashCode = this.__getItemHashCode(item);
    if (this.contains(hashCode)) {
      delete this.__items[hashCode];
      this.__length--;
      return true;
    }
    return false;
  },

  /**
   * @memberOf {util4js.Set}
   * @public
   * @method clear
   */
  clear: function () {
    this.__length = 0;
    this.__items = {}; 
  },

  /**
   * @memberOf {util4js.Set}
   * @public
   * @method contains
   * @param {Object} item
   * @returns {Boolean}
   */
  contains: function (item) {
    var hashCode = this.__getItemHashCode(item);
    return (hashCode in this.__items); 
  },

  /**
   * @memberOf {util4js.Set}
   * @private
   * @method __getItemHashCode
   * @param {Object} item
   * @returns {Number}
   */
  __getItemHashCode: function (item) {
    if (!item) {
      throw new ArgumentNullException({
        paramName: "item"
      });
    }
    if ($is(item, Object)) {
      return item.getHashCode();
    } else if (typeof item === "object") {
      throw new ArgumentException({
        paramName: "item",
        message: "Type is invalid"
      }); 
    }
    return item;
  }

}, Object, ISet);

exports.Set = Set;

