/**
 * @class {util4js.Stack}
 * @extends {util4js.Object}
 * @implements {util4js.IStack}
 */
var Stack = $class({

  /**
   * @memberOf {util4js.Stack}
   * @public
   * @constructor {util4js.Stack}
   * @param {Array} items
   */
  __construct__: function (items) {
    this.__items = [];
    if (items) {
      if (items instanceof Array) {
        for (var i = 0; i < items.length; i++) {
          this.push(items[i]);
        }  
      } else {
        throw new ArgumentException({
          paramName: "items",
          message: "Parameter type is invalid"
        });
      } 
    }
  },

  /**
   * @memberOf {util4js.Stack}
   * @public
   * @method toString
   * @returns {String}
   */
  toString: function () {
    return "util4js.Stack";
  },

  /**
   * @memberOf {util4js.Stack}
   * @public
   * @method iterator
   * @returns {util4js.Iterator}
   */
  iterator: function () {
    return new Iterator(this.__items);
  },

  /**
   * @memberOf {util4js.Stack}
   * @public
   * @property {Number} length
   */
  length: {
    get: function () {
      return this.__items.length;
    }
  },

  /**
   * @memberOf {util4js.Stack}
   * @public
   * @method clear
   */
  clear: function () {
    this.__items = [];
  },

  /**
   * @memberOf {util4js.Stack}
   * @public
   * @method contains
   * @param {Object} item
   */
  contains: function (item) {
    var index;
    if ($is(item, Object)) {
      for (var i = 0; i < this.__items.length; i++) {
        if (Object.equals(this.__items[i], item)) {
          index = i;
          break;
        }
      } 
    } else {
      index = this.__items.indexOf(item);
    }
    return index != -1;
  },

  /**
   * @memberOf {util4js.Stack}
   * @public
   * @method peek
   * @returns {Object}
   */
  peek: function () {
    if (this.__items.length == 0) {
      throw new Exception({
        message: "Stack is empty"
      });
    }
    return this.__items[this.__items.length - 1];
  },

  /**
   * @memberOf {util4js.Stack}
   * @public
   * @method pop
   * @returns {Object}
   */
  pop: function () {
    if (this.__items.length == 0) {
      throw new Exception({
        message: "Stack is empty"
      });
    }
    return this.__items.pop();
  },

  /**
   * @memberOf {util4js.Stack}
   * @public
   * @method push
   * @param {Object} item
   */
  push: function (item) {
    this.__items.push(item);
  }

}, Object, IStack);

exports.Stack = Stack;

